from colorama import Fore

RED = Fore.LIGHTRED_EX
GREEN = Fore.LIGHTGREEN_EX
RESET = Fore.RESET
YELLOW = Fore.LIGHTYELLOW_EX
WHITE = Fore.LIGHTWHITE_EX
BLACK = Fore.LIGHTBLACK_EX
BLUE = Fore.LIGHTBLUE_EX
MAGENTA = Fore.LIGHTMAGENTA_EX
